@@ -1,5 +1,7 @@
-# $Id: crt.py 1347 2015-08-02 14:48:46Z tkeffer $
-# Copyright 2013 Matthew Wall
+# $Id: crt.py 1470 2016-03-28 18:36:17Z mwall $
+# Copyright 2013-2016 Matthew Wall
+# thanks to gary roderick for significant contributions to this code
+
 """Emit loop data to file in Cumulus realtime format.
    http://wiki.sandaysoft.com/a/Realtime.txt
 
@@ -9,6 +11,7 @@ Put this file in bin/user/crt.py, then add this to your weewx.conf:
     unit_system = METRIC # options are US, METRIC, METRICWX
     date_separator = /
     filename = /path/to/realtime.txt
+    none = NULL
 
 [Engine]
     [[Services]]
@@ -19,39 +22,52 @@ If no unit_system is specified, the units will be those of the database.
 Note that most of the code in this file is to calculate/lookup data that
 are not directly provided by weewx in a LOOP packet.
 
-The cumulus 'specification' for realtime.txt is ambiguous:
+The cumulus 'specification' for realtime.txt is ambiguous in places:
 
-  pressure trend interval is not specified, we use 3 hours for pressure
-  temperature trend interval is not specified, we use 3 hours for temperature
-  wind avg speed/dir interval is not specified, we use 10 minutes
-  time zone is not specified, we use utc everywhere
-  pressure is not specified, we use 'barometer' not 'pressure' or 'altimeter'
-
-  how to handle None/NULL is not specified.  we return a NULL when the value
-  is None, but the cumulus specification does not indicate a correct
-  behavior.
+  - pressure trend interval is not specified, we use 3 hours for pressure
+  - temperature trend interval is not specified, we use 3 hours for temperature
+  
+The following assumptions have been made based on the equivalent Cumulus 
+Webtags listed at http://wiki.sandaysoft.com/a/Realtime.txt and 
+  http://wiki.sandaysoft.com/a/Webtags:
+  - wind avg speed/wind avg dir interval is not specified. The equivalent 
+    Cumulus Webtags for wind avg speed and avg wind direction default to 
+    10 minute averages so we use 10 minutes as well.
+  - wind direction bearings in degrees are set from > 0 to 360 with 0
+    indicating calm (refer to the #avgbearing Webtag at
+       http://wiki.sandaysoft.com/a/Webtags) 
+    (this is different to the standard used in weewx)
+  
+The following assumptions have been made based on examination of realtime.txt
+instances from a number of live Cumulus sites:
+  - time zone is not specified, local time is used throughout
+  - how to handle None/NULL is not specified.  Examination of realtime.txt 
+    from a number of live Cumulus sites indicates when there is no wind 
+    (average or gust) wind speeds are set to zero. For other fields that
+    may be None/Null we return the 'none' parameter setting (default = NULL) 
+    from the weewx.conf [CumulusRealTime] section.
+  - ordinal wind directions are set to --- when there is no wind.
 """
 
-# FIXME: is_sunny is ill-defined.  what radiation threshold indicates true?
-# FIXME: is rainYear past 365 days or since 1 jan?
-# FIXME: is rainMonth past 30 days or since 1st of month?
-
 import math
 import time
 import syslog
+from distutils.version import StrictVersion
 
 import weewx
 import weewx.almanac
+import weewx.manager
 import weewx.wxformulas
 import weeutil.weeutil
-import weeutil.Sun
+import weedb
 from weewx.engine import StdService
 
-VERSION = "0.13"
+VERSION = "0.17"
 
-if weewx.__version__ < "3":
-    raise weewx.UnsupportedFeature("weewx 3 is required, found %s" %
-                                   weewx.__version__)
+REQUIRED_WEEWX = "3.2.0"
+if StrictVersion(weewx.__version__) < StrictVersion(REQUIRED_WEEWX):
+    raise weewx.UnsupportedFeature("weewx %s or greater is required, found %s"
+                                   % (REQUIRED_WEEWX, weewx.__version__))
 
 def logmsg(level, msg):
     syslog.syslog(level, 'crt: %s' % msg)
@@ -65,10 +81,8 @@ def loginf(msg):
 def logerr(msg):
     logmsg(syslog.LOG_ERR, msg)
 
-KM_PER_MILE = 1.60934
-
-COMPASS_POINTS = ['N','NNE','NE','ENE','E','ESE','SE','SSE',
-                  'S','SSW','SW','WSW','W','WNW','NW','NNW','N']
+COMPASS_POINTS = ['N', 'NNE', 'NE', 'ENE', 'E', 'ESE', 'SE', 'SSE',
+                  'S', 'SSW', 'SW', 'WSW', 'W', 'WNW', 'NW', 'NNW', 'N']
 
 # map weewx unit names to cumulus unit names
 UNITS_WIND = {'mile_per_hour': 'mph',
@@ -81,83 +95,60 @@ UNITS_TEMP = {'degree_C': 'C',
 UNITS_PRES = {'inHg': 'in',
               'mbar': 'mb',
               'hPa': 'hPa'}
-UNITS_RAIN = {'in': 'in',
+UNITS_RAIN = {'inch': 'in',
               'mm': 'mm'}
+UNITS_ALT = {'foot': 'ft',
+             'meter': 'm'}
 
 def _convert(from_v, from_units, to_units, group):
     vt = (from_v, from_units, group)
     return weewx.units.convert(vt, to_units)[0]
 
-def degreeToCompass(x):
+def clamp_degrees(x):
+    if x is not None:
+        return x if x != 0.0 else 360.0
+    return None
+
+def degree_to_compass(x):
     if x is None:
-        return None
+        return '---'
     idx = int((x + 11.25) / 22.5)
     return COMPASS_POINTS[idx]
 
-# http://www.spc.noaa.gov/faq/tornado/beaufort.html
-def knotToBeaufort(x):
-    if x is None:
-        return None
-    if x < 1:
-        return 0  # calm
-    elif x < 4:
-        return 1  # light air
-    elif x < 7:
-        return 2  # light breeze
-    elif x < 11:
-        return 3  # gentle breeze
-    elif x < 17:
-        return 4  # moderate breeze
-    elif x < 22:
-        return 5  # fresh breeze
-    elif x < 28:
-        return 6  # strong breeze
-    elif x < 34:
-        return 7  # near gale
-    elif x < 41:
-        return 8  # gale
-    elif x < 48:
-        return 9  # strong gale
-    elif x < 56:
-        return 10 # storm
-    elif x < 64:
-        return 11 # violent storm
-    return 12 # hurricane
-
 def get_db_units(dbm):
     val = dbm.getSql("SELECT usUnits FROM %s LIMIT 1" % dbm.table_name)
     return val[0] if val is not None else None
 
-def calcAvgWindSpeed(dbm, ts, interval=600):
+def calc_avg_windspeed(dbm, ts, interval=600):
     sts = ts - interval
     val = dbm.getSql("SELECT AVG(windSpeed) FROM %s "
                      "WHERE dateTime>? AND dateTime<=?" % dbm.table_name,
                      (sts, ts))
     return val[0] if val is not None else None
 
-def calcAvgWindDir(dbm, ts, interval=600):
+def calc_avg_winddir(dbm, ts, interval=600):
     sts = ts - interval
     val = dbm.getSql("SELECT AVG(windDir) FROM %s "
                      "WHERE dateTime>? AND dateTime<=?" % dbm.table_name,
                      (sts, ts))
-    return val[0] if val is not None else None
+    return clamp_degrees(val[0]) if val is not None else None
 
-def calc10MinHighGust(dbm, ts):
+def calc_max_gust_10min(dbm, ts):
     sts = ts - 600
     val = dbm.getSql("SELECT MAX(windGust) FROM %s "
                      "WHERE dateTime>? AND dateTime<=?" % dbm.table_name,
                      (sts, ts))
     return val[0] if val is not None else None
 
-def calc10MinAvgWindDir(dbm, ts):
+def calc_avg_winddir_10min(dbm, ts):
     sts = ts - 600
     val = dbm.getSql("SELECT AVG(windDir) FROM %s "
                      "WHERE dateTime>? AND dateTime<=?" % dbm.table_name,
                      (sts, ts))
-    return val[0] if val is not None else None
+    return clamp_degrees(val[0]) if val is not None else None
 
 # calculate wind run since midnight
-def calcWindRun(dbm, ts, db_us):
+def calc_windrun(dbm, ts, db_us):
     run = 0
     sod_ts = weeutil.weeutil.startOfDay(ts)
     for row in dbm.genSql("SELECT `interval`,windSpeed FROM %s "
@@ -173,7 +164,7 @@ def calcWindRun(dbm, ts, db_us):
     return run
 
 # get trend over past n hours, default to 3 hour window
-def getTrend(label, dbm, ts, n=3):
+def get_trend(label, dbm, ts, n=3):
     lastts = ts - n * 3600
     old_val = dbm.getSql("SELECT %s FROM %s "
                          "WHERE dateTime>? AND dateTime<=?" %
@@ -182,55 +173,55 @@ def getTrend(label, dbm, ts, n=3):
         return None
     return old_val[0]
 
-def calcTrend(newval, oldval):
+def calc_trend(newval, oldval):
     if newval is None or oldval is None:
         return None
     return newval - oldval
 
-def calcRainHour(dbm, ts):
+def calc_rain_hour(dbm, ts):
     sts = ts - 3600
     val = dbm.getSql("SELECT SUM(rain) FROM %s "
                      "WHERE dateTime>? AND dateTime<=?" % dbm.table_name,
                      (sts, ts))
     return val[0] if val is not None else None
 
-def calcRainMonth(dbm, ts):
+def calc_rain_month(dbm, ts):
     span = weeutil.weeutil.archiveMonthSpan(ts)
     val = dbm.getSql("SELECT SUM(rain) FROM %s "
-                         "WHERE dateTime>? AND dateTime<=?" % dbm.table_name,
-                         (span.start, ts))
+                     "WHERE dateTime>? AND dateTime<=?" % dbm.table_name,
+                     (span.start, ts))
     return val[0] if val is not None else None
 
-def calcRainYear(dbm, ts):
+def calc_rain_year(dbm, ts):
     span = weeutil.weeutil.archiveYearSpan(ts)
     val = dbm.getSql("SELECT SUM(rain) FROM %s "
                      "WHERE dateTime>? AND dateTime<=?" % dbm.table_name,
                      (span.start, ts))
     return val[0] if val is not None else None
 
-def calcRainYesterday(dbm, ts):
+def calc_rain_yesterday(dbm, ts):
     ts = weeutil.weeutil.startOfDay(ts)
-    sts = ts - 3600*24;
+    sts = ts - 3600 * 24
     val = dbm.getSql("SELECT SUM(rain) FROM %s "
                      "WHERE dateTime>? AND dateTime<=?" % dbm.table_name,
                      (sts, ts))
     return val[0] if val is not None else None
 
-def calcDayRain(dbm, ts):
+def calc_rain_day(dbm, ts):
     sts = weeutil.weeutil.startOfDay(ts)
     val = dbm.getSql("SELECT SUM(rain) FROM %s "
                      "WHERE dateTime>=? AND dateTime<=?" % dbm.table_name, 
                      (sts, ts))
     return val[0] if val is not None else None
 
-def calcETToday(dbm, ts):
+def calc_ET_today(dbm, ts):
     sts = weeutil.weeutil.startOfDay(ts)
     val = dbm.getSql("SELECT SUM(ET) FROM %s "
                      "WHERE dateTime>? AND dateTime<=?" % dbm.table_name,
                      (sts, ts))
     return val[0] if val is not None else None
 
-def calcMinMax(label, dbm, ts, minmax='MAX'):
+def calc_minmax(label, dbm, ts, minmax='MAX'):
     sts = weeutil.weeutil.startOfDay(ts)
     val = dbm.getSql("SELECT %s(%s) FROM %s "
                      "WHERE dateTime>=? AND dateTime<=?" %
@@ -243,65 +234,34 @@ def calcMinMax(label, dbm, ts, minmax='MAX'):
     if t is None:
         return None, None
     tstr = time.strftime("%H:%M", time.gmtime(t[0]))
-    return val[0],tstr
+    return val[0], tstr
 
-def calcHumidex(t_C, dewpoint_K):
-    if t_C is None or dewpoint_K is None:
-        return None
-    v = 1/273.16
-    v -= 1/dewpoint_K
-    v *= 5417.7530
-    v = math.exp(v)
-    v *= 6.11
-    v -= 10
-    v *= 0.5555
-    return t_C + v
-
-def calcIsDaylight(ts_utc, lat, lon):
-    x = weeutil.weeutil.startOfDayUTC(ts_utc)
-    x_tt = time.gmtime(x)
-    y, m, d = x_tt[:3]
-    (sunrise_utc, sunset_utc) = weeutil.Sun.sunRiseSet(y, m, d, lon, lat)
-    if sunrise_utc < ts_utc < sunset_utc:
+def calc_is_daylight(alm):
+    sunrise = alm.sunrise.raw
+    sunset = alm.sunset.raw
+    if sunrise < alm.time_ts < sunset:
         return 1
     return 0
 
-def calcDaylightHours(ts_utc, lat, lon):
-    x = weeutil.weeutil.startOfDayUTC(ts_utc)
-    x_tt = time.gmtime(x)
-    y, m, d = x_tt[:3]
-    (sunrise_utc, sunset_utc) = weeutil.Sun.sunRiseSet(y, m, d, lon, lat)
-    if ts_utc <= sunrise_utc:
+def calc_daylight_hours(alm):
+    sunrise = alm.sunrise.raw
+    sunset = alm.sunset.raw
+    if alm.time_ts <= sunrise:
         return 0
-    elif ts_utc < sunset_utc:
-        return (ts_utc - sunrise_utc) / 60
-    return (sunset_utc - sunrise_utc) / 60
+    elif alm.time_ts < sunset:
+        return (alm.time_ts - sunrise) / 3600.0
+    return (sunset - sunrise) / 3600.0
 
-def calcCloudBase(outTemp_F, dewpoint_F, altitude_foot):
-    if outTemp_F is None or dewpoint_F is None:
-        return None
-    return int((outTemp_F - dewpoint_F) / 4.4 * 1000) + altitude_foot
-
-def calcMaxRad(alm, altitude_m):
-    try:
-        rs_el = alm.sun.alt
-        rs_R = alm.sun.earth_distance
-        rs_atc = 0.8
-        rs_z = altitude_m
-        rs_r0 = 1367
-        sinal = math.sin(rs_el)
-        if sinal < 0:
-            return 0
-        al = math.asin(sinal)
-        a0 = al * 57.2957795
-        rm = math.pow((288-0.0065*rs_z)/288,5.256)/(sinal+0.15*math.pow(a0+3.885,-1.253))
-        rs_toa = rs_r0 * sinal / (rs_R * rs_R)
-        return rs_toa * math.pow(rs_atc, rm)
-    except ValueError, e:
-        logdbg("cannot calculate max solar radiation: %s" % e)
+def calc_is_sunny(rad, max_rad, threshold):
+    if not rad or not max_rad:
         return 0
+    if rad <= threshold * max_rad:
+        return 0
+    return 1
 
-# this depends on the weather station.  many stations have more than one.
+# indication of sensor contact depens on the weather station.  if the station
+# has more than one indicator, then indicate failure if contact is lost with
+# any one of them.
 #
 # Vantage
 #   packet['rxCheckPercent'] == 0
@@ -325,7 +285,7 @@ def calcMaxRad(alm, altitude_m):
 # WS23xx
 #   packet['cn'] == 'lost'
 #
-def lostSensorContact(packet):
+def lost_sensor_contact(packet):
     if 'rxCheckPercent' in packet and packet['rxCheckPercent'] == 0:
         return 1
     if 'cn' in packet and packet['cn'] == 'lost':
@@ -341,6 +301,58 @@ def lostSensorContact(packet):
         return 1
     return 0
 
+class ZambrettiForecast():
+    DEFAULT_FORECAST_BINDING = 'forecast_binding'
+    DEFAULT_BINDING_DICT = {
+        'database': 'forecast_sqlite',
+        'manager': 'weewx.manager.Manager',
+        'table_name': 'archive',
+        'schema': 'user.forecast.schema'}
+
+    def __init__(self, config_dict):
+        self.forecasting_installed = False
+        self.db_max_tries = 3
+        self.db_retry_wait = 3
+        try:
+            self.dbm_dict = weewx.manager.get_manager_dict(
+                config_dict['DataBindings'],
+                config_dict['Databases'],
+                ZambrettiForecast.DEFAULT_FORECAST_BINDING,
+                default_binding_dict=ZambrettiForecast.DEFAULT_BINDING_DICT)
+            weewx.manager.open_manager(self.dbm_dict)
+            self.forecasting_installed = True
+        except (weedb.DatabaseError, weewx.UnsupportedFeature, KeyError):
+            pass
+
+    def is_installed(self):
+        return self.forecasting_installed
+
+    def get_zambretti_code(self):
+        if not self.forecasting_installed:
+            return 0
+
+        # FIXME: add api to forecast instead of doing all the work here
+        with weewx.manager.open_manager(self.dbm_dict) as dbm:
+            sql = "select dateTime,zcode from %s where method = 'Zambretti' order by dateTime desc limit 1" % dbm.table_name
+#            sql = "select zcode from %s where method = 'Zambretti' and dateTime = (select max(dateTime) from %s where method = 'Zambretti')" % (dbm.table_name, dbm.table_name)
+            for count in range(self.db_max_tries):
+                try:
+                    record = dbm.getSql(sql)
+                    if record is not None:
+                        return ZambrettiForecast.alpha_to_number(record[1])
+                except Exception, e:
+                    logerr('get zambretti failed (attempt %d of %d): %s' %
+                           ((count + 1), self.db_max_tries, e))
+                    logdbg('waiting %d seconds before retry' %
+                           self.db_retry_wait)
+                    time.sleep(self.db_retry_wait)
+        return 0
+
+    # given a zambretti letter code A-Z, convert to digit 1-26
+    @staticmethod
+    def alpha_to_number(x):
+        return ord(x) - 64
+
 
 class CumulusRealTime(StdService):
 
@@ -356,6 +368,10 @@ class CumulusRealTime(StdService):
         d = config_dict.get('CumulusRealTime', {})
         self.filename = d.get('filename', '/var/tmp/realtime.txt')
         self.datesep = d.get('date_separator', '/')
+        self.sunny_threshold = float(d.get('sunny_threshold', 0.75))
+        nonesub = d.get('none', 'NULL')
+        self.nonesub = nonesub if nonesub is not '' else 'NULL'
+        loginf("'None' values will be displayed as %s" % self.nonesub)
         binding = d.get('binding', 'loop').lower()
         us = None
         us_label = d.get('unit_system', None)
@@ -367,6 +383,9 @@ class CumulusRealTime(StdService):
                 logerr("unknown unit_system %s" % us_label)
         self.unit_system = us
 
+        self.forecast = ZambrettiForecast(config_dict)
+        loginf("zambretti is installed: %s" % self.forecast.is_installed())
+
         if binding == 'loop':
             self.bind(weewx.NEW_LOOP_PACKET, self.handle_new_loop)
         else:
@@ -397,6 +416,8 @@ class CumulusRealTime(StdService):
             f.write("\n")
 
     def _cvt(self, from_v, to_us, obs, group):
+        if self.db_us is None:
+            return None
         from_units = weewx.units.getStandardUnitType(self.db_us, obs)[0]
         to_units = weewx.units.getStandardUnitType(to_us, obs)[0]
         return _convert(from_v, from_units, to_units, group)
@@ -418,127 +439,88 @@ class CumulusRealTime(StdService):
         t_u = weewx.units.getStandardUnitType(pkt_us, 'outTemp')[0]
         r_u = weewx.units.getStandardUnitType(pkt_us, 'rain')[0]
         ws_u = weewx.units.getStandardUnitType(pkt_us, 'windSpeed')[0]
+        alt_u = weewx.units.getStandardUnitType(pkt_us, 'altitude')[0]
 
         data = dict(packet)
         data['windSpeed_avg'] = self._cvt(
-            calcAvgWindSpeed(dbm, ts), pkt_us, 'windSpeed', 'group_speed')
-        data['windDir_compass'] = degreeToCompass(packet.get('windDir'))
+            calc_avg_windspeed(dbm, ts), pkt_us, 'windSpeed', 'group_speed')
+        data['cumulus_windDir'] = clamp_degrees(packet.get('windDir'))
+        data['windDir_compass'] = degree_to_compass(packet.get('windDir'))
         v = _convert(packet.get('windSpeed'), ws_u, 'knot', 'group_speed')
-        data['windSpeed_beaufort'] = knotToBeaufort(v)
+        data['windSpeed_beaufort'] = weewx.wxformulas.beaufort(v)
         data['units_wind'] = UNITS_WIND.get(ws_u, ws_u)
         data['units_temperature'] = UNITS_TEMP.get(t_u, t_u)
         data['units_pressure'] = UNITS_PRES.get(p_u, p_u)
         data['units_rain'] = UNITS_RAIN.get(r_u, r_u)
-        # weewx does not know distance, so we have to convert this directly
-        wr = calcWindRun(dbm, ts, self.db_us)
-        if self.db_us != pkt_us:
-            if self.db_us == weewx.US:
-                if pkt_us == weewx.METRIC:
-                    wr *= KM_PER_MILE
-                elif pkt_us == weewx.METRICWX:
-                    wr *= KM_PER_MILE * 1000
-            elif self.db_us == weewx.METRIC:
-                if pkt_us == weewx.US:
-                    wr /= KM_PER_MILE
-                elif pkt_us == weewx.METRICWX:
-                    wr *= 1000
-            elif self.db_us == weewx.METRICWX:
-                if pkt_us == weewx.US:
-                    wr /= (KM_PER_MILE * 1000)
-                elif pkt_us == weewx.METRIC:
-                    wr /= 1000
-        # wind run is mile, kilometer, or meter
-        data['wind_run'] = wr
+        wr = calc_windrun(dbm, ts, self.db_us)
+        data['windrun'] = self._cvt(wr, pkt_us, 'windrun', 'group_distance')
         p1 = packet.get('barometer')
-        p2 = getTrend('barometer', dbm, ts)
+        p2 = get_trend('barometer', dbm, ts)
         p2 = self._cvt(p2, pkt_us, 'barometer', 'group_pressure')
-        data['pressure_trend'] = calcTrend(p1, p2)
+        data['pressure_trend'] = calc_trend(p1, p2)
         t1 = packet.get('outTemp')
-        t2 = getTrend('outTemp', dbm, ts)
+        t2 = get_trend('outTemp', dbm, ts)
         t2 = self._cvt(t2, pkt_us, 'outTemp', 'group_temperature')
-        data['temperature_trend'] = calcTrend(t1, t2)
+        data['temperature_trend'] = calc_trend(t1, t2)
         data['rain_month'] = self._cvt(
-            calcRainMonth(dbm, ts), pkt_us, 'rain', 'group_rain')
+            calc_rain_month(dbm, ts), pkt_us, 'rain', 'group_rain')
         data['rain_year'] = self._cvt(
-            calcRainYear(dbm, ts), pkt_us, 'rain', 'group_rain')
+            calc_rain_year(dbm, ts), pkt_us, 'rain', 'group_rain')
         data['rain_yesterday'] = self._cvt(
-            calcRainYesterday(dbm, ts), pkt_us, 'rain', 'group_rain')
+            calc_rain_yesterday(dbm, ts), pkt_us, 'rain', 'group_rain')
         data['dayRain'] = self._cvt(
-            calcDayRain(dbm, ts), pkt_us, 'rain', 'group_rain')
-        v,t = calcMinMax('outTemp', dbm, ts, 'MAX')
+            calc_rain_day(dbm, ts), pkt_us, 'rain', 'group_rain')
+        v, t = calc_minmax('outTemp', dbm, ts, 'MAX')
         data['outTemp_max'] = self._cvt(
             v, pkt_us, 'outTemp', 'group_temperature')
         data['outTemp_max_time'] = t
-        v,t = calcMinMax('outTemp', dbm, ts, 'MIN')
+        v, t = calc_minmax('outTemp', dbm, ts, 'MIN')
         data['outTemp_min'] = self._cvt(
             v, pkt_us, 'outTemp', 'group_temperature')
         data['outTemp_min_time'] = t
-        v,t = calcMinMax('windSpeed', dbm, ts, 'MAX')
+        v, t = calc_minmax('windSpeed', dbm, ts, 'MAX')
         data['windSpeed_max'] = self._cvt(
             v, pkt_us, 'windSpeed', 'group_speed')
         data['windSpeed_max_time'] = t
-        v,t = calcMinMax('windGust', dbm, ts, 'MAX')
+        v, t = calc_minmax('windGust', dbm, ts, 'MAX')
         data['windGust_max'] = self._cvt(
             v, pkt_us, 'windGust', 'group_speed')
         data['windGust_max_time'] = t
-        v,t = calcMinMax('barometer', dbm, ts, 'MAX')
+        v, t = calc_minmax('barometer', dbm, ts, 'MAX')
         data['pressure_max'] = self._cvt(
             v, pkt_us, 'barometer', 'group_pressure')
         data['pressure_max_time'] = t
-        v,t = calcMinMax('barometer', dbm, ts, 'MIN')
+        v, t = calc_minmax('barometer', dbm, ts, 'MIN')
         data['pressure_min'] = self._cvt(
             v, pkt_us, 'barometer', 'group_pressure')
         data['pressure_min_time'] = t
         data['10min_high_gust'] = self._cvt(
-            calc10MinHighGust(dbm, ts), pkt_us, 'windSpeed', 'group_speed')
+            calc_max_gust_10min(dbm, ts), pkt_us, 'windSpeed', 'group_speed')
 
-        t_F = _convert(packet.get('outTemp'),
-                       t_u, 'degree_F', 'group_temperature')
-        x = weewx.wxformulas.heatindexF(t_F, packet.get('outHumidity'))
-        data['heatindex'] = _convert(x, 'degree_F', t_u, 'group_temperature')
+        data['ET_today'] = calc_ET_today(dbm, ts)
+        data['10min_avg_wind_bearing'] = calc_avg_winddir_10min(dbm, ts)
+        data['rain_hour'] = self._cvt(
+            calc_rain_hour(dbm, ts), pkt_us, 'rain', 'group_rain')
+        data['lost_sensors_contact'] = lost_sensor_contact(packet)
+        data['avg_wind_dir'] = degree_to_compass(data['10min_avg_wind_bearing'])
+        data['units_cloudbase'] = UNITS_ALT.get(alt_u)
         t_C = _convert(packet.get('outTemp'),
                        t_u, 'degree_C', 'group_temperature')
-        dp_C = _convert(packet.get('dewpoint'),
-                        t_u, 'degree_C', 'group_temperature')
-        dp_K = dp_C + 273.15 if dp_C is not None else None
-        v = calcHumidex(t_C, dp_K);
-        data['humidex'] = _convert(v, 'degree_C', t_u, 'group_temperature')
-        data['ET_today'] = calcETToday(dbm, ts)
-        data['10min_avg_wind_bearing'] = calc10MinAvgWindDir(dbm, ts)
-        data['rain_hour'] = self._cvt(
-            calcRainHour(dbm, ts), pkt_us, 'rain', 'group_rain')
-        data['is_daylight'] = calcIsDaylight(ts, self.latitude, self.longitude)
-        data['lost_sensors_contact'] = lostSensorContact(packet)
-        data['avg_wind_dir'] = degreeToCompass(calcAvgWindDir(dbm, ts))
-        dp_F = _convert(packet.get('dewpoint'),
-                        t_u, 'degree_F', 'group_temperature')
-        cb_f = calcCloudBase(t_F, dp_F, self.altitude_ft)
-        ut = weewx.units.getStandardUnitType(pkt_us, 'altitude')
-        vt = (cb_f, "foot", "group_altitude")
-        data['cloud_base'] = weewx.units.convert(vt, ut[0])[0]
-        data['units_cloud_base'] = ut[0]
-        if (v < 50):
-            ws_mph = _convert(packet.get('windSpeed'),
-                              ws_u, 'mile_per_hour', 'group_speed')
-            x = weewx.wxformulas.windchillF(t_F, ws_mph)
-            v = _convert(x, 'degree_F', t_u, 'group_temperature')
-        else:
-            v = data['heatindex']
-        data['apparent_temperature'] = v
-        data['sunshine_hours'] = calcDaylightHours(
-            ts, self.latitude, self.longitude)
-        alt_m = self.altitude_ft * 0.3048
-        p_mbar = _convert(packet.get('barometer'), p_u, 'mbar', 'group_pressure')
+        alt_m = _convert(self.altitude_ft, 'foot', 'meter', 'group_altitude')
+        p_mbar = _convert(packet.get('barometer'), p_u, 
+                          'mbar', 'group_pressure')
         alm = weewx.almanac.Almanac(ts, self.latitude, self.longitude, alt_m,
                                     t_C, p_mbar)
-        data['max_rad'] = calcMaxRad(alm, alt_m)
+        data['is_daylight'] = calc_is_daylight(alm)
+        data['sunshine_hours'] = calc_daylight_hours(alm)
         data['is_sunny'] = data['is_daylight']
+        data['zambretti_code'] = self.forecast.get_zambretti_code()
         return data
 
     def format(self, data, label, places=None):
         value = data.get(label)
         if value is None:
-            value = 'NULL'
+            value = self.nonesub
         elif places is not None:
             try:
                 v = float(value)
@@ -553,16 +535,16 @@ class CumulusRealTime(StdService):
     def create_realtime_string(self, data):
         fields = []
         datefmt = "%%d%s%%m%s%%y" % (self.datesep, self.datesep)
-        tstr = time.strftime(datefmt, time.gmtime(data['dateTime']))
+        tstr = time.strftime(datefmt, time.localtime(data['dateTime']))
         fields.append(tstr)                                           # 1
-        tstr = time.strftime("%H:%M:%S", time.gmtime(data['dateTime']))
+        tstr = time.strftime("%H:%M:%S", time.localtime(data['dateTime']))
         fields.append(tstr)                                           # 2
         fields.append(self.format(data, 'outTemp', 1))                # 3
         fields.append(self.format(data, 'outHumidity', 0))            # 4
         fields.append(self.format(data, 'dewpoint', 1))               # 5
         fields.append(self.format(data, 'windSpeed_avg', 1))          # 6  *
         fields.append(self.format(data, 'windSpeed', 1))              # 7
-        fields.append(self.format(data, 'windDir', 0))                # 8
+        fields.append(self.format(data, 'cumulus_windDir', 0))        # 8
         fields.append(self.format(data, 'rainRate', 1))               # 9
         fields.append(self.format(data, 'dayRain', 1))                # 10
         fields.append(self.format(data, 'barometer', 1))              # 11
@@ -572,7 +554,7 @@ class CumulusRealTime(StdService):
         fields.append(self.format(data, 'units_temperature'))         # 15 *
         fields.append(self.format(data, 'units_pressure'))            # 16 *
         fields.append(self.format(data, 'units_rain'))                # 17 *
-        fields.append(self.format(data, 'wind_run', 1))               # 18 *
+        fields.append(self.format(data, 'windrun', 1))               # 18 *
         fields.append(self.format(data, 'pressure_trend', 1))         # 19 *
         fields.append(self.format(data, 'rain_month', 1))             # 20 *
         fields.append(self.format(data, 'rain_year', 1))              # 21 *
@@ -600,17 +582,17 @@ class CumulusRealTime(StdService):
         fields.append(self.format(data, 'humidex', 1))                # 43 *
         fields.append(self.format(data, 'UV', 0))                     # 44
         fields.append(self.format(data, 'ET_today', 1))               # 45 *
-        fields.append(self.format(data, 'radiation'))                 # 46
+        fields.append(self.format(data, 'radiation', 0))              # 46
         fields.append(self.format(data, '10min_avg_wind_bearing', 0)) # 47 *
         fields.append(self.format(data, 'rain_hour'))                 # 48 *
-        fields.append('0')                                            # 49
+        fields.append(self.format(data, 'zambretti_code'))            # 49 *
         fields.append(self.format(data, 'is_daylight'))               # 50 *
         fields.append(self.format(data, 'lost_sensors_contact'))      # 51 *
         fields.append(self.format(data, 'avg_wind_dir'))              # 52 *
-        fields.append(self.format(data, 'cloud_base', 0))             # 53 *
-        fields.append(self.format(data, 'units_cloud_base'))          # 54 *
-        fields.append(self.format(data, 'apparent_temperature', 1))   # 55 *
+        fields.append(self.format(data, 'cloudbase', 0))              # 53 *
+        fields.append(self.format(data, 'units_cloudbase'))           # 54 *
+        fields.append(self.format(data, 'appTemp', 1))                # 55 *
         fields.append(self.format(data, 'sunshine_hours', 1))         # 56 *
-        fields.append(self.format(data, 'max_rad', 1))                # 57 ##
+        fields.append(self.format(data, 'maxSolarRad', 1))            # 57
         fields.append(self.format(data, 'is_sunny'))                  # 58 *
         return ' '.join(fields)
